//
// AminoAcidProperty.h
//
// This file contains the BTL_AminoAcidProperty functor.
//
// These classes are part of the Bioinformatics Template Library (BTL).
//
// Copyright (C) 1997,1998 Birkbeck College, Malet Street, London WC1E 7HX, U.K.
// (classlib@mail.cryst.bbk.ac.uk)
// 
// This library is free software; you can
// redistribute it and/or modify it under the terms of
// the GNU Library General Public License as published by the Free
// Software Foundation; either version 2 of the License, or (at your
// option) any later version.  This library is distributed in the hope
// that it will be useful, but WITHOUT ANY WARRANTY; without even the
// implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
// PURPOSE.  See the GNU Library General Public License for more details.
// You should have received a copy of the GNU Library General Public
// License along with this library; if not, write to the Free Software
// Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
///////////////////////////////////////////////////////////////////////////

#if !defined (BTL_AMINOACIDPROPERTY_H)
#define BTL_AMINOACIDPROPERTY_H

#if !defined(SGI_CC)
#include <map>
#else
#include <map.h>
#endif

#if !defined(SGI_CC) && !defined(GCC_272)
using namespace std;
#endif

enum BTL_Scale { KyteHydropathy, MillerAccessibility, DarbyVolume,
                 RadzickaHydrophilicity, RadzickaHydrophobicity };

/**#: [Description ="This class defines a function object that will, given an
    amino acid single letter code with return a property value for that residue
    type."]
    [Summary = "functor that returns amino acid property values"] 
    [Authors = "W.R.Pitt"]
    [Files = "<A HREF=./btl/AminoAcidProperty.h>AminoAcidProperty.h</A>,
              <A HREF=./btl/AminoAcidProperty.cpp>AminoAcidProperty.cpp</A>"]
    [Dependencies="none"]

*/

class BTL_AminoAcidProperty
{
private:
    
    typedef map<char,float,less<char> > value_type;
    value_type data;

	    	/**#: [Hidden] */
    static void Use_DC_Volume(value_type& values);
	    	/**#: [Hidden] */
    static void Use_MJLC_AccessibleSurfaceArea(value_type& values);
	    	/**#: [Hidden] */
    static void Use_RW_Hydrophilicity(value_type& values);
	    	/**#: [Hidden] */
    static void Use_KD_Hydropathy(value_type& values);
	    	/**#: [Hidden] */
    static void Use_RW_Hydrophobicity(value_type& values);
        
public:

	    	/**#: [Description="Default constructor. The default property
		type is KyteHydropathy."] */
    BTL_AminoAcidProperty();
    
	    	/**#: [Description="Construct function object which will return
		property values of a given type."] */
    BTL_AminoAcidProperty(const BTL_Scale whatScale);

	    	/**#: [Description="Operator that will return a property value
		given a single letter amino acid code."] */
    float 
    operator()(const char aacode) const;
};

#endif
