//
// PIRProcessor.cpp
//
// This file contains the definitions for the PIRProcessor class.
//
// These classes are part of the Bioinformatics Template Library (BTL).
//
// Copyright (C) 1997 Birkbeck College, Malet Street, London WC1E 7HX, U.K.
// (classlib@mail.cryst.bbk.ac.uk)
// 
// This library is free software; you can
// redistribute it and/or modify it under the terms of
// the GNU Library General Public License as published by the Free
// Software Foundation; either version 2 of the License, or (at your
// option) any later version.  This library is distributed in the hope
// that it will be useful, but WITHOUT ANY WARRANTY; without even the
// implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
// PURPOSE.  See the GNU Library General Public License for more details.
// You should have received a copy of the GNU Library General Public
// License along with this library; if not, write to the Free Software
// Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
///////////////////////////////////////////////////////////////////////////

#include "btl/PIRProcessor.h"
 
void BTL_PIRProcessor::OpenFile(const char* fileName, ifstream& fileStream)
{
    fileStream.open(fileName);

    // check that file can be opened
    if (!fileStream.good())
    {
        cerr << "\n\nFile " << fileName << " cannot be opened" << endl;
        exit(1);
    }
    
}

int BTL_PIRProcessor::HowManySeqs(const char* fileName, ifstream& fileStream)
{
    int nseqs=0;
    const short MAXLINE = 512;
    char line[MAXLINE];
    while(fileStream.getline(line,MAXLINE))
     	if( line[0] == '>' ) nseqs++;
    fileStream.close();
    fileStream.open(fileName);
    
    cout << nseqs << " sequences found in " << fileName << "\n\n";
    return nseqs;
}    

void BTL_PIRProcessor::ReadFile(const char *fileName)
{ 
    ifstream fin;
    OpenFile(fileName,fin);

    int nseqs = HowManySeqs(fileName,fin);
    
    //
    // Extract sequences from file 
    //
    const short MAXLINE = 512;
    char line[MAXLINE];
    int iseq = 0;

    // Read each sequence in turn
    //
    while(iseq<nseqs)
    {
        char c;

        // Read until the beginning of a sequence header
        //
        do
        {
            if (!fin.getline(line,MAXLINE)) break;
            c = line[0];
        }
        while(c != '>');
            
        if (c!='>') cerr << "\n!!!Error reading " << fileName 
                         << ": initial > expected.\n" << endl;

    	// Skip comment field
    	//
        if (!fin.getline(line,MAXLINE)) break;    
    
    	// Read until the end of the sequence
    	//
    	string seq;

    	do
        {
            if (!fin.getline(line,MAXLINE)) break;
            for (int i=0; i<MAXLINE; i++)
            {
                c=line[i];
                if(c == '\n' || c == '\0' || c == '*')
                    break;
                if (c != ' ')
                    seq += c;
            }
    	}
        while(c!='*');
        
        if (c!='*') cerr << "\n!!!Error reading " << fileName 
                         << ": terminal * expected.\n" << endl;

    	if (!seq.empty()) seqs.push_back(seq);

        iseq++;    
    }
    fin.close();
}
