//
// XYZProcessor.h
//
// This file contains declarations for the XYZProcessor class.
//
// These classes are part of the Bioinformatics Template Library (BTL).
//
// Copyright (C) 1997,1998 Birkbeck College, Malet Street, London WC1E 7HX, U.K.
// (classlib@mail.cryst.bbk.ac.uk)
// 
// This library is free software; you can
// redistribute it and/or modify it under the terms of
// the GNU Library General Public License as published by the Free
// Software Foundation; either version 2 of the License, or (at your
// option) any later version.  This library is distributed in the hope
// that it will be useful, but WITHOUT ANY WARRANTY; without even the
// implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
// PURPOSE.  See the GNU Library General Public License for more details.
// You should have received a copy of the GNU Library General Public
// License along with this library; if not, write to the Free Software
// Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
///////////////////////////////////////////////////////////////////////////

#if !defined(BTL_XYZPROCESSOR_H)
#define BTL_XYZPROCESSOR_H

#include "btl/BTL.h"

#if !defined(SGI_CC)
#include <string>
#include <vector>
#else
#include <mstring.h>
#include <vector.h>
#endif

#include <strstream.h>
#include <iomanip.h>
#include <fstream.h>

#if !defined(SGI_CC) && !defined(GCC_272)
using namespace std;
#endif

/**#: [Description ="This is simple class for reading xyz format files.
     It is really only included in the library so that real data can be used
     in the test programs."]
    [Summary = "reads xyz format coordinate files and puts the data in vectors"]
    [Friends = "an output operator"]
    [Authors = "W.R.Pitt"]
    [Files = "<A HREF=./btl/XYZProcessor.h>XYZProcessor.h</A>"]
    [Dependencies="None"]
*/

class BTL_XYZProcessor
{
    typedef vector<float>   CoordStore;
    typedef vector<char>    ElementStore;

private:

    CoordStore	    coords;
    ElementStore    elements;
    string  	    title;

	    	/**#: [Hidden] */
    void
    OpenFile(const char* fileName, ifstream& fileStream);


public:

	    	/**#: [Description="Default constructor (does nothing)"] */

    BTL_XYZProcessor() {}

	    	/**#: [Description="Returns true if no coordinates have been read"] */
    bool
    empty() const { return coords.empty(); }

	    	/**#: [Description="Read an xyz format file with the given name"] */
    void
    ReadFile(const char *fileName);

	    	/**#: [Description="Return a reference to the read coordinates in a
                STL vector of float. Coordinates are stored in the following
                order: x,y,z,x,y,z,x,..."] */
    CoordStore&
    Coords() { return coords; }

	    	/**#: [Description="Return a reference to the read atomic elements in a
                STL vector of char. Each element is stored as 2 chars."] */
    ElementStore&
    Elements() { return elements; }

    friend ostream&
    operator<<(ostream& os, const BTL_XYZProcessor& x);
};

#endif
