

#if !defined(SGI_CC)
#include <complex>
#else
#include <complex.h>
#endif
#include <fstream.h>

#include "btl/FFT.h"
#include "btl/PPMProcessor.h"


int main(int argc, char* argv[])
{
    if (argc != 3)
    {
    	cerr << "Usage: FFTdemo inputFile (1|0)" << endl;
    	exit(1);
    }
    
    BTL_PPMProcessor p;
    p.ReadFile(argv[1]);
    
    char *direction = argv[2];
    bool forward = atoi(direction);
    if (forward) cerr << "Forward\n";
    else cerr << "Backward\n";

    typedef vector< complex<double> > Container;
    typedef Container::iterator iterator;

    unsigned int width  = p.ReadWidth(), 
                 height = p.ReadHeight(),
                 size   = width*height;
    
    vector<double> rawData = p.ReadImageData();

    Container imageData;
    imageData.reserve(size);
    
    vector<double>::const_iterator iter;
    for (iter=rawData.begin(); iter!=rawData.end(); iter+=3)
    {
    	complex<double> temp(*iter,0.0);
    	imageData.push_back(temp);
    }

    Container transformedData(size);
    BTL_FFT<Container::iterator>::Transform(imageData.begin(), 
                                            transformedData.begin(), 
                     			    width, height, forward);

    vector<double> product;
    product.reserve(size*3);
    
    for (iterator i=transformedData.begin(); i!=transformedData.end(); i++)
    {
    	double rl = i->real(), im = i->imag(), prod = rl*rl + im*im,
	       logprod = log1p(prod);
    	product.push_back(logprod); // R
    	product.push_back(logprod); // G
    	product.push_back(logprod); // B
    }
    	
    p.WriteImageData(product,width,height);
    
    ofstream fout1("out1.ppm");
    fout1 << p;
    fout1.close();
    

    BTL_FFT<Container::iterator>::Transform(transformedData.begin(), 
                                            imageData.begin(), 
                    			    width, height, !forward);

    product.erase(product.begin(),product.end());
    for (iterator i=imageData.begin(); i!=imageData.end(); i++)
    {
    	double normal = norm(*i);
    	product.push_back(normal);
    	product.push_back(normal);
    	product.push_back(normal);
    }
            
    p.WriteImageData(product,width,height);

    ofstream fout2("out2.ppm");
    fout2 << p;
    fout2.close();

    return 0;
}    
