// Program to test the function AreNotEqual(const& T, const& T)

#include "btl/BTL.h"

#if !defined(SGI_CC)
#include <string>
#else
#include <mstring.h>
#endif

#include "btl/NumericUtilities.h"

#if !defined(SGI_CC) && !defined(GCC_272)
using namespace std;
#endif

//.............................................................................

void TestResults(const string& testName, const unsigned int& error)
{
    cout << ">>>>> ERROR = " 
    	 << error
    	 << " : "
    	 << testName 
    	 << " <<<<<\n" 
    	 << endl;
}

int main()
{
    unsigned int error;
    
    // Both inputs zero
    error = BTL_NumericUtilities<double>::AreNotEqual(0.0,0.0);
    TestResults("0.0,0.0 (should be 0)",error);

    // One input zero
    error = BTL_NumericUtilities<double>::AreNotEqual(0.000000000001,0.0);
    TestResults("0.0,0.000000000001 (should be 15)",error);

    // Two tiny doubles
    error = BTL_NumericUtilities<double>::AreNotEqual(1E-78,1E-78);
    TestResults("1E-78,1E-78 (should be zero)",error);

    // Two tiny floats
    error = BTL_NumericUtilities<float>::AreNotEqual(1E-78,1E-78);
    TestResults("(float) 1E-78, (float) 1E-78 (should be zero)",error);

    // A float and a double - PRODUCES COMPILATION ERROR
//     error = AreNotEqual((double) 1E-78, (float) 1E-78);
//     TestResults("(double) 1E-78, (float) 1E-78 (should be zero)",error);

    // A very large and a very small long double
    error = BTL_NumericUtilities<long double>::AreNotEqual(1E123L,1E-123L);
    TestResults("1E123L,1E-123L (should = LDBL_DIG)",error);

    // A very large and a very small double
    error = BTL_NumericUtilities<double>::AreNotEqual(1E123,1E-123);
    TestResults("1E123,1E-123 (should = DBL_DIG)",error);
    
    // A very large and a very small float
    error = BTL_NumericUtilities<float>::AreNotEqual(1E15,1E-15);
    TestResults("(float) 1E15, (float) 1E-15 (should = FLT_DIG)",error);

    // Two doubles different in one decimal place
    error = BTL_NumericUtilities<double>::AreNotEqual(555555.0, 555554.0);
    TestResults("555555.0, 555554.0 (should be DBL_DIG-5 )",error);

    // Two floats different in one decimal place
    error = BTL_NumericUtilities<float>::AreNotEqual(555555.0,555554.0);
    TestResults("555555.0,555554.0 (should be FLT_DIG-5 )", error);

    // A positive double and a negative double
    error = BTL_NumericUtilities<double>::AreNotEqual(-1.0, 1.0);
    TestResults("-1.0, 1.0 (should = DBL_DIG)",error);


//     error = AreNotEqual((double) 0.0,(double) 0.000000000001);
//     TestResults("(double) 0.0,(double) 0.000000000001",error);
// 
//     error = AreNotEqual((float) 0.0,(float) 0.000000000001);
//     TestResults("(float) 0.0,(float) 0.000000000001",error);
// 
//     float a=555555.0, b=555554.0;
//     error = AreNotEqual(a,b);
//     TestResults("555555.0,555554.0",error);
// 
//     a = 5.55555; b=5.55554;
//     error = AreNotEqual(a,b);
//     TestResults("5.55555,5.55554",error);
// 
//     a = -5.55555; b=-5.55554;
//     error = AreNotEqual(a,b);
//     TestResults("-5.55555,-5.55554",error);
// 
//     a = 555555.0E-15; b=555554.0E-15;
//     error = AreNotEqual(a,b);
//     TestResults("555555.0E-15,555554.0E-15",error);
// 
//     a = -555555.0; b=555554.0;
//     error = AreNotEqual(a,b);
//     TestResults("-555555.0,555554.0",error);
// 
//     a = 555555.0; b=55555.4;
//     error = AreNotEqual(a,b);
//     TestResults("555555.0,55555.4",error);
// 
//     a = -555555.0; b=-55555.4;
//     error = AreNotEqual(a,b);
//     TestResults("-555555.0,-55555.4",error);
// 
//     a = 555555.0E-15; b=555554.0E-16;
//     error = AreNotEqual(a,b);
//     TestResults("555555.0E-15,555554.0E-16",error);   
//    
//     double  big1 = 10000008.8, big2 = 10000000.0,
//            calcDiff = big1 - big2, trueDiff = 8.8;
//     
//     error = AreNotEqual(calcDiff,trueDiff);
//     TestResults("8.8, 100008.8 - 100000.0",error);
//     
//     float c = 99999999.9, d = 99999999.9;
//     error = AreNotEqual(c,d);
//     TestResults("99999999.9, 99999999.9",error);

    return 0;
}
